import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JPanel;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;

/**
 * Ein Dialogfenster fuer den Chatclient.
 * Vorgesehen sind ein Textfeld zur Eingabe und ein
 * Textbereich zur Ausgabe.
 * 
 * @author Q11
 * @version 2015-06-15
 */
public class Chatfenster
{
    /*
     * Das einzige verfuegbare Chatfenster.
     */
    private static Chatfenster chatfensterSingleton;
    
    /*
     * Die Komponenten des Dialogfensters.
     */
    private JFrame fenster;
    private JPanel dialograum;
    private JTextField eingabe;
    private JTextPane ausgabe;
    private JScrollPane ausgabeRolle;
    
    /*
     * Die Stilarten bei der Ausgabe.
     */
    private Style eingabeStil;
    private Style ausgabeStil;
    
    /**
     * Gib das einzige verfuegbare Chatfenster zurueck (also den
     * "chatfensterSingleton"). Wenn das Chatfenster noch nicht
     * erzeugt ist, erzeuge es.
     */
    public static Chatfenster gibChatfenster ()
    {
        if (null == chatfensterSingleton)
        {
            chatfensterSingleton = new Chatfenster ();
        }
        
        return chatfensterSingleton;
    }
    
    /**
     * Konstruktor.
     * Erzeuge das einzige verfuegbare Chatfenster.
     */
    private Chatfenster ()
    {
        eingabe = new JTextField ();
        ausgabe = new JTextPane ();
        
        ausgabe.setPreferredSize (new Dimension (400, 400));
        ausgabe.setEditable (false);
        
        eingabeStil = ausgabe.addStyle ("Eingabe", null);
        StyleConstants.setForeground (eingabeStil, Color.red);
        
        ausgabeStil = ausgabe.addStyle ("Ausgabe", null);
        StyleConstants.setForeground (ausgabeStil, Color.black);
        
        ausgabeRolle = new JScrollPane (ausgabe);
        
        dialograum = new JPanel (new BorderLayout ());
        dialograum.add (eingabe, BorderLayout.NORTH);
        dialograum.add (ausgabeRolle, BorderLayout.CENTER);

        fenster = new JFrame ("Chatfenster");
        fenster.setDefaultCloseOperation (JFrame.DO_NOTHING_ON_CLOSE);
        fenster.getContentPane ().add (dialograum);
        fenster.pack ();
        
        int bildschirmbreite = (int) Toolkit.getDefaultToolkit().getScreenSize().getWidth ();
        int bildschirmhoehe  = (int) Toolkit.getDefaultToolkit().getScreenSize().getHeight ();
        int fensterbreite    = fenster.getWidth ();
        int fensterhoehe     = fenster.getHeight ();
        
        fenster.setBounds (bildschirmbreite - fensterbreite, 0, fensterbreite, fensterhoehe);
        fenster.setVisible (true);
    }
    
    /**
     * Nenne die Eingabe des Chatfensters.
     */
    public JTextField gibEingabe ()
    {
        return eingabe;
    }
    
    /**
     * Nenne die Ausgabe des Chatfensters.
     */
    public JTextPane gibAusgabe ()
    {
        return ausgabe;
    }
    
    /**
     * Nenne den Eingabestil.
     */
    public Style gibEingabestil ()
    {
        return eingabeStil;
    }
    
    /**
     * Nenne den Ausgabestil.
     */
    public Style gibAusgabestil ()
    {
        return ausgabeStil;
    }
    
    /**
     * Aendere die Farbe des Eingabestils.
     */
    public void setzeEingabestilFarbe (Color gewuenschteFarbe)
    {
        StyleConstants.setForeground (eingabeStil, gewuenschteFarbe);
    }
    
    /**
     * Springe zum Ende der Ausgabe.
     */
    public void springeZumEndeDerAusgabe ()
    {
        ausgabe.setCaretPosition (ausgabe.getStyledDocument ().getLength ());
    }
}
