import java.awt.Color;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.Socket;
import javax.swing.text.StyledDocument;

/**
 * Ein Empfangsprozess fr den Spielclient.
 * 
 * @author Q11
 * @version 2015-06-16
 */
public class ClientEmpfangsProzess implements Runnable
{
    private Socket meinSocket;
    private Spiel meinSpiel;

    /**
     * Konstruktor.
     * Verbinde Dich an den angegebenen Socket und starte ein
     * neues Spiel.
     */
    public ClientEmpfangsProzess (Socket s, ClientSendeProzess e)
    { 
        meinSocket = s;
        meinSpiel  = new Spiel (60, 35);

        e.setzeSpiel (meinSpiel);
        Leinwand.gibLeinwand ().gibFenster ().addKeyListener (e);
        Leinwand.gibLeinwand ().gibZeichenflaeche ().addMouseListener (e);
        Chatfenster.gibChatfenster ().gibEingabe ().addActionListener (e);
    }

    /**
     * Ablaufplan.
     * Lies und echoe vom Server, bis er "Bye." sendet.
     */
    public void run ()
    { 
        try 
        {
            /*
             * Datenstrom vom Server.
             */
            BufferedReader liesVomServer   = new BufferedReader (new InputStreamReader (meinSocket.getInputStream ()));
            
            /*
             * Ausgabezeile.
             */
            String serverZeile;
            
            /*
             * Ausgabedokument im Chatfenster.
             */
            StyledDocument zieldokument = Chatfenster.gibChatfenster ().gibAusgabe ().getStyledDocument (); 

            /*
             * Chatte, solange der Server sendet und solange er sich
             * nicht verabschiedet.
             */  
            serverZeile = liesVomServer.readLine ();

            while (serverZeile != null && !serverZeile.equals ("Bye."))
            {
                /*
                 * Gib die Serverzeile im Chatfenster aus.
                 */
                try
                {
                    zieldokument.insertString (zieldokument.getLength (), serverZeile + "\n", Chatfenster.gibChatfenster ().gibAusgabestil ());
                }
                catch (Exception fehler)
                {
                }
                
                Chatfenster.gibChatfenster ().springeZumEndeDerAusgabe ();
                
                /*
                 * Untersuche die Serverzeile auf wichtige Befehle hin.
                 */
                if (serverZeile.startsWith ("Name: "))
                {
                    meinSpiel.setzeMeinenNamen (serverZeile.substring (6));
                }
                else if (serverZeile.startsWith ("Farben: "))
                {
                    String[] bestandteile = serverZeile.split (" ");

                    try
                    {
                        int meinRot   = Integer.parseInt (bestandteile[1]);
                        int meinGruen = Integer.parseInt (bestandteile[2]);
                        int meinBlau  = Integer.parseInt (bestandteile[3]);

                        meinSpiel.setzeMeineFarbe (new Color (meinRot, meinGruen, meinBlau));
                        
                        Chatfenster.gibChatfenster ().setzeEingabestilFarbe (new Color (meinRot, meinGruen, meinBlau));
                    }
                    catch (Exception fehler)
                    {
                    }
                }
                else if (serverZeile.startsWith ("Playing against "))
                {
                    String[] bestandteile = serverZeile.split (" ");

                    meinSpiel.setzeGegnerNamen (bestandteile [2]);

                    try
                    {
                        int meinRot   = Integer.parseInt (bestandteile[3]);
                        int meinGruen = Integer.parseInt (bestandteile[4]);
                        int meinBlau  = Integer.parseInt (bestandteile[5]);

                        meinSpiel.setzeGegnerFarbe (new Color (meinRot, meinGruen, meinBlau));
                    }
                    catch (Exception fehler)
                    {
                    }
                    
                    meinSpiel.fangeAn ();
                }
                else if (serverZeile.startsWith ("moved hoch "))
                {
                    String[] bestandteile = serverZeile.split (" ");
                    
                    try
                    {
                        int x = Integer.parseInt (bestandteile [3]);
                        int y = Integer.parseInt (bestandteile [4]);
                        
                        String name = bestandteile [2];
                        
                        meinSpiel.faerbeHochbalken (name, x, y);
                    }
                    catch (Exception fehler)
                    {
                        /*
                         * Wenn sich jemand den Nick "moved" gibt, knnen dessen Lebenszeichen
                         * flschlicherweise als Spielzge interpretiert werden. Das fhrt zu
                         * Fehlermeldungen, die wir einfach ignorieren.
                         */
                    }
                }
                else if (serverZeile.startsWith ("moved quer "))
                {
                    String[] bestandteile = serverZeile.split (" ");
                    
                    try
                    {
                        int x = Integer.parseInt (bestandteile [3]);
                        int y = Integer.parseInt (bestandteile [4]);
                        
                        String name = bestandteile [2];
                        
                        meinSpiel.faerbeQuerbalken (name, x, y);
                    }
                    catch (Exception fehler)
                    {
                        /*
                         * Wenn sich jemand den Nick "moved" gibt, knnen dessen Lebenszeichen
                         * flschlicherweise als Spielzge interpretiert werden. Das fhrt zu
                         * Fehlermeldungen, die wir einfach ignorieren.
                         */
                    }
                }
                else if (serverZeile.equals ("Done."))
                {
                    meinSpiel.hoereAuf ();
                }

                serverZeile = liesVomServer.readLine ();
            }

            /*
             * Schliee den Datenstrom und den Socket.
             */
            liesVomServer.close ();
            meinSocket.close ();
        }
        catch (IOException fehler) 
        {
            System.err.println ("Keine Verbindung beim Empfangen.");
        }
    }

    /**
     * Nenne das Spiel, das mir zugeordnet ist.
     */
    public Spiel gibSpiel ()
    { 
        return meinSpiel;
    }
}