import java.awt.Color;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.awt.Shape;

/**
 * Zeichnet ein Rechteck auf die Leinwand.
 * 
 * @author Bernd Gramlich
 * @version 2015-05-13
 */
public class Rechteck
{
    // Attribute
    private int positionX;
    private int positionY;
    private int breite;
    private int hoehe;    
    private Color farbe;
    private boolean sichtbar;

    /**
     * Konstruktor der Klasse Rechteck.
     * Erzeugt ein neues Rechteck in Standardgroesse und Standardfarbe.
     */
    public Rechteck ()
    {
        hoehe    = 20;
        breite   = 30;
        farbe    = Color.red;
        sichtbar = false;
    }
    
	/**
	 * Mache dieses Rechteck sichtbar.
	 */
	public void werdeSichtbar ()
	{
		sichtbar = true;
		zeichne ();
	}

	/**
	 * Mache dieses Rechteck unsichtbar.
	 */
	public void werdeUnsichtbar ()
	{
		loesche ();
		sichtbar = false;
	}
	
    /**
	 * Aendere die Hoehe dieses Rechtecks.
	 */
	public void setzeHoehe (int neueHoehe)
	{
		hoehe = neueHoehe;
		zeichne ();
	}
	
	/**
	 * Aendere die Breite dieses Rechtecks.
	 */
	public void setzeBreite (int neueBreite)
	{
		breite = neueBreite;
		zeichne ();
	}
		
	/**
	 * Aendere die Farbe dieses Rechtecks in 'neueFarbe'.
	 */
	public void setzeFarbe (Color neueFarbe)
	{
		farbe = neueFarbe;
		zeichne ();
	}
		
	/**
	 * Nenne die Farbe dieses Rechtecks.
	 */
	public Color gibFarbe ()
	{
		return farbe;
	}
	
	/**
	 * Pruefe, ob ein Punkt in diesem Rechteck enthalten ist.
	 */
	public boolean enthaelt (Point2D punkt)
	{
		Shape form = Leinwand.gibLeinwand ().gibForm (this);
		
		if (null != form)
		{
		    return form.contains (punkt);
		}
		else
		{
		    return false;
		}
	}
	
	/**
	 * Aendere die Position dieses Rechtecks.
	 */
	public void setzePosition (int neuePositionX, int neuePositionY)
	{
		positionX = neuePositionX;
		positionY = neuePositionY;
		zeichne ();
	}
		
	/**
	 * Zeichne dieses Rechteck auf die Leinwand.
	 */
	private void zeichne ()
	{
		if (sichtbar)
		{
			Leinwand leinwand = Leinwand.gibLeinwand ();
			leinwand.merkeZumZeichnenVor (this, farbe, new Rectangle (positionX,
					positionY, breite, hoehe));
		}
	}

	/**
	 * Loesche dieses Rechteck von der Leinwand.
	 */
	private void loesche ()
	{
		if (sichtbar)
		{
			Leinwand leinwand = Leinwand.gibLeinwand ();
			leinwand.entferne (this);
		}
	}
}